# BetterFileUploader.pl
# Better File Uploader plugin for Movable Type
# http://www.danandsherree.com/2005/09/13/better_file_uploader.php
# by Dan Wolfgang
# http://www.danandsherree.com
# last modified 10/9/2005

use strict;
package MT::Plugin::BetterFileUploader;

use vars qw( $VERSION );
$VERSION = '1.06';

require MT::Plugin;
require MT;

# These are the default upload locations and other variables.
my $path = 'upload/';
my $thumbnail_size = 128;

my $settings;
eval {
	$settings = new MT::PluginSettings([
		['system_override', { Default => 0 }],
		['path', { Default => $path }],
		['path_append', { Default => 1 }],
		['archive_or_site', { Default => 1 }],
		['new_entry_type', { Default => 0 }],
		['thumbnail_size_type', { Default => 1 }],
		['thumbnail_size_longest', { Default => $thumbnail_size }],
		['thumbnail_size_fixed', { Default => $thumbnail_size }],
		['thumbnail_fixed_dimension', { Default => 0 }],
		['gallery', { Default => 1 }],
		['gallery_extended', { Default => 0 }]
	]);
};


my $plugin = MT::Plugin->new({
	name => "Better File Uploader",
	description => 'Set default File Upload options.',
	doc_link => 'http://www.danandsherree.com/2005/09/13/better_file_uploader.php',
	author_name => 'Dan Wolfgang',
	author_link => 'http://www.danandsherree.com/',
	version => $VERSION,
	config_template => \&_config_template,
	settings => $settings
});
MT->add_plugin($plugin);

MT->add_callback('bigpapi::template::upload::3.2x', 9, $plugin, \&_upload_template);
MT->add_callback('bigpapi::param::upload', 9, $plugin, \&_upload_param);
MT->add_callback('bigpapi::template::upload_complete::3.2x', 9, $plugin, \&_upload_complete_template);
MT->add_callback('bigpapi::param::upload_complete', 9, $plugin, \&_upload_complete_param);
MT->add_callback('bigpapi::template::show_upload_html::3.2x', 9, $plugin, \&_show_upload_html_template);



# Configuration options
sub _config_template {
	my ($plugin, $param, $scope) = @_;
	my $html = '';
	my $article;
	if ($scope eq 'system') {
		$html = <<HTML;
<div class="setting">
<div class="field">
<p><input value="1" type="checkbox" name="system_override" id="system_override" <TMPL_IF NAME=SYSTEM_OVERRIDE>checked="checked"</TMPL_IF> />
<MT_TRANS phrase="Override weblog-specific settings with the settings below:"></p>
</div>
</div>
HTML
    }

    $html .= <<HTML;
<div>
<p><strong>Default Upload Options</strong></p>
<div style="padding-left: 1.5em;">

<div class="setting">
<div class="field">
<p>Default upload directory: <input type="text" name="path" id="path" value="<TMPL_VAR NAME=PATH>" /><br />
(Be sure to append a trailing slash to the path. Examples: <code>upload/</code>, <code>images/</code>, <code>something/else/</code>.)</p>
</div>
<div class="field">
<p><label><input value="1" type="checkbox" name="path_append" id="path_append" <TMPL_IF NAME=PATH_APPEND>checked</TMPL_IF> /> Append <code>[current year]/[current month]/</code> to default upload directory.</label></p>
</div>
</div>

<div class="setting">
<div class="field">
<p>By default, upload to:</p>
<p><label><input type="radio" name="archive_or_site" value="1" <TMPL_IF NAME=ARCHIVE_OR_SITE>checked</TMPL_IF> /> Local archive path.</label></p>
<p><label><input type="radio" name="archive_or_site" value="0" <TMPL_UNLESS NAME=ARCHIVE_OR_SITE>checked</TMPL_UNLESS> /> Local site path.</label></p>
</div>
</div>

<div class="setting">
<div class="field">
<p>By default, publish as:</p>
<p><label><input type="radio" name="new_entry_type" value="1" <TMPL_IF NAME=NEW_ENTRY_TYPE>checked</TMPL_IF> /> Create a new entry using this uploaded file.</label></p>
<p><label><input type="radio" name="new_entry_type" value="0" <TMPL_UNLESS NAME=NEW_ENTRY_TYPE>checked</TMPL_UNLESS> /> Show me the HTML.</label></p>
</div>
</div>

<div class="setting">
<div class="field">
<p>After uploading an image, a thumbnail can be built. How should the thumbnail size be calculated?</p>

<p><label><input type="radio" name="thumbnail_size_type" id="thumbnail_size_type" value="1" <TMPL_IF NAME=THUMBNAIL_SIZE_TYPE>checked</TMPL_IF> /> The longest side should be </label><label><input type="text" name="thumbnail_size_longest" id="thumbnail_size_longest" value="<TMPL_VAR NAME=THUMBNAIL_SIZE_LONGEST>" size="3" /> pixels long.</label></p>

<p><label><input type="radio" name="thumbnail_size_type" id="thumbnail_size_type" value="0" <TMPL_UNLESS NAME=THUMBNAIL_SIZE_TYPE>checked</TMPL_UNLESS> /> The thumbnail should be a fixed size.</label></p>
	<div style="padding-left: 1.5em;">
	<p>The thumbnail should be <input type="text" name="thumbnail_size_fixed" id="thumbnail_size_fixed" value="<TMPL_VAR NAME=THUMBNAIL_SIZE_FIXED>" size="3" /> pixels <label><input type="radio" name="thumbnail_fixed_dimension" id="thumbnail_fixed_dimension" value="0" <TMPL_UNLESS NAME=THUMBNAIL_FIXED_DIMENSION>checked</TMPL_UNLESS> /> wide</label> <label><input type="radio" name="thumbnail_fixed_dimension" id="thumbnail_fixed_dimension" value="1" <TMPL_IF NAME=THUMBNAIL_FIXED_DIMENSION>checked</TMPL_IF> /> tall</label>.</p>
	</div>
</div>
</div>

</div>
</div>


<div>
<p><strong>Photo Gallery Creation Options</strong></p>
<div style="padding-left: 1.5em;">

<div class="setting">
<div class="field">
<p><label><input value="1" type="checkbox" name="gallery" id="gallery" <TMPL_IF NAME=GALLERY>checked</TMPL_IF> /> Enable easier photo gallery creation mode.</label></p>
</div>
<div class="field" style="padding-left: 1.5em;">
<p><label><input type="radio" name="gallery_extended" value="0" <TMPL_UNLESS NAME=GALLERY_EXTENDED>checked</TMPL_UNLESS> /> Place the photo path in the Entry Body field.</label></p>
<p><label><input type="radio" name="gallery_extended" value="1" <TMPL_IF NAME=GALLERY_EXTENDED>checked</TMPL_IF> /> Place the photo path in the Extended Entry field.</label></p>
</div>
</div>

</div>
</div>
HTML
	return $html;
}






sub blog_config {
	my ($blog_id) = @_;
	my $sys = $plugin->get_config_hash('system');
	if ($sys->{'system_override'}) {
		return $sys;
	} else {
		return $plugin->get_config_hash('blog:' . $blog_id);
	}
}









# Update upload.tmpl with the upload path/year/month stuff.
sub _upload_template {
	my ($cb, $app, $template) = @_;

	# Replace Local Site Path and Local Archive Path
	my $old1 = qq{<TMPL_IF NAME=ENABLE_ARCHIVE_PATHS>
<div class="field">
<input type="radio" name="site_path" value="0" checked="checked" />
&lt;<MT_TRANS phrase="Local Archive Path">&gt;/
<input name="extra_path_archive" /><br />

<input type="radio" name="site_path" value="1" />
&lt;<MT_TRANS phrase="Local Site Path">&gt;/
<input name="extra_path_site" />
</div>
<TMPL_ELSE>
<input type="hidden" name="site_path" value="1" />
<p>&lt;<MT_TRANS phrase="Local Site Path">&gt; / <input name="extra_path_site" /></p>
</TMPL_IF>
};
	$old1 = quotemeta($old1);
	my $new1 = <<HTML;
<div class="field">
<TMPL_IF NAME=ENABLE_ARCHIVE_PATHS>
<input type="radio" name="site_path" value="0" <TMPL_IF NAME=ARCHIVE_OR_SITE>checked</TMPL_IF> />
&lt;<MT_TRANS phrase="Local Archive Path">&gt;/ <input name="extra_path_archive" value="<TMPL_VAR NAME=PATH>" /><br />

<input type="radio" name="site_path" value="1" <TMPL_UNLESS NAME=ARCHIVE_OR_SITE>checked</TMPL_UNLESS> />
&lt;<MT_TRANS phrase="Local Site Path">&gt;/ <input name="extra_path_site" value="<TMPL_VAR NAME=PATH>" />
<TMPL_ELSE>
<input type="radio" name="site_path" value="1" checked />
&lt;<MT_TRANS phrase="Local Site Path">&gt;/ <input name="extra_path_site" value="<TMPL_VAR NAME=PATH>" />
</TMPL_IF>
</div>
HTML
	$$template =~ s/$old1/$new1/;

	# Add the JavaScript
	my $old4 = qq{</form>};
	$old4 = quotemeta($old4);
	my $new4 = <<HTML;
</form>

<script type="text/javascript" language="JavaScript">
var date = new Date();
var day = date.getDate();
var m = date.getMonth() + 1;
var month = (m < 10) ? '0' + m : m;
var year = date.getFullYear();
<TMPL_IF NAME=ENABLE_ARCHIVE_PATHS>
<TMPL_VAR NAME=PATH_APPEND_ARCHIVE>
<TMPL_ELSE>
<TMPL_VAR NAME=PATH_APPEND_SITE>
</TMPL_IF>
</script>
HTML
	$$template =~ s/$old4/$new4/;
}


sub _upload_param {
	my ($cb, $app, $param) = @_;

	my $config = blog_config($param->{'blog_id'});

	# Add the year/month/ bit, if necessary.
	my $path_append = $config->{'path_append'};
	my $new = '';
	my $new2 = '';
	if ($path_append == 1) {
		$path = $config->{'path'};
		$new = <<HTML;
document.forms[0].extra_path_archive.value = '$path' + year + '/' + month + '/';
document.forms[0].extra_path_site.value = '$path' + year + '/' + month + '/';
HTML
		$new2 = <<HTML;
document.forms[0].extra_path_site.value = '$path' + year + '/' + month + '/';
HTML
	}

	$param->{PATH} = $config->{'path'};
	$param->{PATH_APPEND_ARCHIVE} = $new;
	$param->{PATH_APPEND_SITE} = $new2;
	$param->{ARCHIVE_OR_SITE} = $config->{'archive_or_site'};
}









# Update upload_complete.tmpl with the Photo Gallery button and related code
sub _upload_complete_template {
	my ($cb, $app, $template) = @_;

	# Replace the JavaScript pieces
	my $oldpaths = qq{function doShowHTML (f, mode) {
    var url = '<TMPL_VAR NAME=SCRIPT_URL>?__mode=show_upload_html&blog_id=<TMPL_VAR NAME=BLOG_ID>&url=<TMPL_VAR NAME=URL ESCAPE=URL>&site_path=<TMPL_VAR NAME=SITE_PATH>&fname=<TMPL_VAR NAME=FNAME ESCAPE=URL>&' + mode;
<TMPL_IF NAME=DO_THUMB>
    if (f.thumb.checked)
        url += '&thumb=1&' + calcGeometry(f);
</TMPL_IF>
    window.location = url;
}

function doStartEntry (f, mode) {
    var url = '<TMPL_VAR NAME=SCRIPT_URL>?__mode=start_upload_entry&blog_id=<TMPL_VAR NAME=BLOG_ID>&url=<TMPL_VAR NAME=URL ESCAPE=URL>&site_path=<TMPL_VAR NAME=SITE_PATH>&fname=<TMPL_VAR NAME=FNAME ESCAPE=URL>&' + mode;
<TMPL_IF NAME=DO_THUMB>
    if (f.thumb.checked)
        url += '&thumb=1&' + calcGeometry(f);
</TMPL_IF>
    window.opener.location = url;
    window.close();
}};
	$oldpaths = quotemeta($oldpaths);
	my $newpaths = <<HTML;
function doShowHTML (f, mode) {
    var url = '<TMPL_VAR NAME=SCRIPT_DIR>?__mode=show_upload_html&blog_id=<TMPL_VAR NAME=BLOG_ID>&url=<TMPL_VAR NAME=URL ESCAPE=URL>&site_path=<TMPL_VAR NAME=SITE_PATH>&fname=<TMPL_VAR NAME=FNAME ESCAPE=URL>&' + mode;
<TMPL_IF NAME=DO_THUMB>
    if (f.thumb.checked)
        url += '&thumb=1&' + calcGeometry(f);
</TMPL_IF>
    window.location = url;
}

function doStartEntry (f, mode) {
    var url = '<TMPL_VAR NAME=SCRIPT_DIR>?__mode=start_upload_entry&blog_id=<TMPL_VAR NAME=BLOG_ID>&url=<TMPL_VAR NAME=URL ESCAPE=URL>&site_path=<TMPL_VAR NAME=SITE_PATH>&fname=<TMPL_VAR NAME=FNAME ESCAPE=URL>&' + mode;
<TMPL_IF NAME=DO_THUMB>
    if (f.thumb.checked)
        url += '&thumb=1&' + calcGeometry(f);
</TMPL_IF>
    window.opener.location = url;
    window.close();
}
HTML
	$$template =~ s/$oldpaths/$newpaths/;


	my $old = qq{function adjustWidthHeight (f, width) {
    if (!f.constrain.checked) return;
    var w = f.thumb_width;
    var wt = f.thumb_width_type;
    var wf = f.full_width.value;
    var h = f.thumb_height;
    var ht = f.thumb_height_type;
    var hf = f.full_height.value;
    var pct;
    var s = width ? wt : ht;
    if (s.options[s.selectedIndex].value == 'pixels')
        pct = width ? (w.value / wf) : (h.value / hf);
    else
        pct = width ? w.value / 100 : h.value / 100;
    var e = width ? h : w;
    var s = width ? ht : wt;
    var full = width ? hf : wf;
    if (s.options[s.selectedIndex].value == 'pixels')
        e.value = Math.floor(pct * full);
    else
        e.value = Math.floor(pct * 100);
    return true;
}};
	$old = quotemeta($old);
	my $new = <<HTML;
function adjustWidthHeight (f, width) {
    if (!f.constrain.checked) return;
    var w = f.thumb_width;
    var wt = f.thumb_width_type;
    var wf = f.full_width.value;
    var h = f.thumb_height;
    var ht = f.thumb_height_type;
    var hf = f.full_height.value;
    var pct;
    var s = width ? wt : ht;
    if (s.options[s.selectedIndex].value == 'pixels')
        pct = width ? (w.value / wf) : (h.value / hf);
    else
        pct = width ? w.value / 100 : h.value / 100;
    var e = width ? h : w;
    var s = width ? ht : wt;
    var full = width ? hf : wf;
    if (s.options[s.selectedIndex].value == 'pixels')
        e.value = Math.floor(pct * full);
    else
        e.value = Math.floor(pct * 100);
    return true;
}

function FixThumbnailDimensions(f) {
<TMPL_IF NAME=THUMBNAIL_SIZE_TYPE>
	if (f.full_width.value >= f.full_height.value) {
	    f.thumb_width.value = '<TMPL_VAR NAME=THUMBNAIL_SIZE_LONGEST>';
	    adjustWidthHeight(f, 1);
	} else {
	    f.thumb_height.value = '<TMPL_VAR NAME=THUMBNAIL_SIZE_LONGEST>';
	    adjustWidthHeight(f, 0);
	}
</TMPL_IF>
<TMPL_UNLESS NAME=THUMBNAIL_SIZE_TYPE>
	<TMPL_UNLESS NAME=THUMBNAIL_FIXED_DIMENSION>
	    f.thumb_width.value = '<TMPL_VAR NAME=THUMBNAIL_SIZE_FIXED>';
	    adjustWidthHeight(f, 1);
	</TMPL_UNLESS>
	<TMPL_IF NAME=THUMBNAIL_FIXED_DIMENSION>
	    f.thumb_height.value = '<TMPL_VAR NAME=THUMBNAIL_SIZE_FIXED>';
	    adjustWidthHeight(f, 0);
	</TMPL_IF>
</TMPL_UNLESS>
}

function doPhotoGallery (f, mode) {
    f.thumb.checked = true;
    FixThumbnailDimensions(f);
    var url = '<TMPL_VAR NAME=SCRIPT_DIR>?__mode=start_upload_<TMPL_IF NAME=GALLERY_EXTENDED>extended_</TMPL_IF>entry&blog_id=<TMPL_VAR NAME=BLOG_ID>&url=<TMPL_VAR NAME=URL ESCAPE=URL>&site_path=<TMPL_VAR NAME=SITE_PATH>&fname=<TMPL_VAR NAME=FNAME ESCAPE=URL>&' + mode;
<TMPL_IF NAME=DO_THUMB>
    if (f.thumb.checked)
        url += '&thumb=1&' + calcGeometry(f);
</TMPL_IF>

    window.opener.location = url;
    window.close();
}
HTML
	$$template =~ s/$old/$new/;

	# Replace the form elements
	my $oldaction = qq{<form action="<TMPL_VAR NAME=SCRIPT_URL>" onsubmit="return false;">};
	$oldaction = quotemeta($oldaction);
	my $newaction = <<HTML;
<form action="<TMPL_VAR NAME=SCRIPT_DIR>" onsubmit="return false;">
HTML
	$$template =~ s/$oldaction/$newaction/;

	# Set the default new entry/show HTML type.
	my $old2 = qq{<label><input type="radio" name="new_entry" value="1" checked="checked" /> <MT_TRANS phrase="Create a new entry using this uploaded file"></label> <a href="#" onclick="return openManual('file_upload', 'creating_a_new_entry')" class="help">?</a><br />
<label><input type="radio" name="new_entry" value="0"> <MT_TRANS phrase="Show me the HTML"></label>};
	$old2 = quotemeta($old2);
	my $new2 = <<HTML;
<label><input type="radio" name="new_entry" value="1" <TMPL_IF NAME=NEW_ENTRY_TYPE>checked</TMPL_IF> /> <MT_TRANS phrase="Create a new entry using this uploaded file"></label> <a href="#" onclick="return openManual('file_upload', 'creating_a_new_entry')" class="help">?</a><br />
<label><input type="radio" name="new_entry" value="0" <TMPL_UNLESS NAME=NEW_ENTRY_TYPE>checked</TMPL_UNLESS> /> <MT_TRANS phrase="Show me the HTML"></label>
HTML
	$$template =~ s/$old2/$new2/;

	my $old3 = qq{<input type="checkbox" name="thumb" value="1">};
	$old3 = quotemeta($old3);
	my $new3 = <<HTML;
<input type="checkbox" name="thumb" value="1" onclick="FixThumbnailDimensions(this.form);">
HTML
	$$template =~ s/$old3/$new3/;

	my $old4 = qq{<p><MT_TRANS phrase="Would you like this file to be a:"></p>};
	$old4 = quotemeta($old4);
	my $new4 = <<HTML;
<TMPL_IF NAME=IS_IMAGE>
<TMPL_VAR NAME=GALLERY_TEXT>
</TMPL_IF>

<p><MT_TRANS phrase="Would you like this file to be a:"></p>
HTML
	$$template =~ s/$old4/$new4/;

	my $old5 = qq{<input type="button" onclick="doClick(this.form, 'popup=1&amp;width=<TMPL_VAR NAME=WIDTH>&amp;height=<TMPL_VAR NAME=HEIGHT>&amp;image_type=<TMPL_VAR NAME=IMAGE_TYPE>')" value="<MT_TRANS phrase="Popup Image">" />
<input type="button" onclick="doClick(this.form, 'include=1&width=<TMPL_VAR NAME=WIDTH>&height=<TMPL_VAR NAME=HEIGHT>&image_type=<TMPL_VAR NAME=IMAGE_TYPE>')" value="<MT_TRANS phrase="Embedded Image">" />};
	$old5 = quotemeta($old5);
	my $new5 = <<HTML;
<input type="button" onclick="doClick(this.form, 'popup=1&amp;width=<TMPL_VAR NAME=WIDTH>&amp;height=<TMPL_VAR NAME=HEIGHT>&amp;image_type=<TMPL_VAR NAME=IMAGE_TYPE>')" value="<MT_TRANS phrase="Popup Image">" />
<input type="button" onclick="doClick(this.form, 'include=1&width=<TMPL_VAR NAME=WIDTH>&height=<TMPL_VAR NAME=HEIGHT>&image_type=<TMPL_VAR NAME=IMAGE_TYPE>')" value="<MT_TRANS phrase="Embedded Image">" />
<TMPL_VAR NAME=GALLERY_BUTTON>
HTML
	$$template =~ s/$old5/$new5/;
}

sub _upload_complete_param {
   	my ($cb, $app, $param) = @_;

	$param->{SCRIPT_DIR} = $app->path . "plugins/BetterFileUploader/BetterFileUploader.cgi";

	my $config = blog_config($param->{'blog_id'});

	$param->{NEW_ENTRY_TYPE} = $config->{'new_entry_type'};
	$param->{THUMBNAIL_SIZE_TYPE} = $config->{'thumbnail_size_type'};
	$param->{THUMBNAIL_SIZE_LONGEST} = $config->{'thumbnail_size_longest'};
	$param->{THUMBNAIL_SIZE_FIXED} = $config->{'thumbnail_size_fixed'};
	$param->{THUMBNAIL_FIXED_DIMENSION} = $config->{'thumbnail_fixed_dimension'};
	$param->{GALLERY_EXTENDED} = $config->{'gallery_extended'};

	# Display the Photo Gallery button?
	if ($config->{'gallery'} == 1) {
		$param->{GALLERY_TEXT} = qq{<p>The above options are ignored if you click "For Photo Gallery," below.</p>};
		$param->{GALLERY_BUTTON} = qq{<input type="button" onclick="doPhotoGallery(this.form, 'photogallery=1&width=<TMPL_VAR NAME=WIDTH>&height=<TMPL_VAR NAME=HEIGHT>&image_type=<TMPL_VAR NAME=IMAGE_TYPE>')" value="For Photo Gallery" />};
	} else {
		$param->{GALLERY_TEXT} = '';
		$param->{GALLERY_BUTTON} = '';
	}
}



sub _show_upload_html_template {
	my ($cb, $app, $template) = @_;

	my $old6 = qq{<textarea name="html" id="html" rows="10" style="width: 100%;"><TMPL_VAR NAME=UPLOAD_HTML ESCAPE=HTML></textarea>};
	$old6 = quotemeta($old6);
	my $new6 = <<HTML;
<textarea name="html" id="html" rows="10" style="width: 100%;" onClick="javascript:this.form.html.focus();this.form.html.select();"><TMPL_VAR NAME=UPLOAD_HTML ESCAPE=HTML></textarea>
HTML
	$$template =~ s/$old6/$new6/;
}

1;
